"""Courses.
"""
import grok
from zope.interface import implementedBy
from zope.component.interfaces import IFactory
from waeup.interfaces import ICourse
from waeup.viewlets import MainArea, Index, Manage, FormWrapMixin
from waeup.utils.helpers import getName

class Course(grok.Model):
    grok.implements(ICourse)

    def __init__(self,
                 title=u'Unnamed Course',
                 code=u'NA',
                 level=None,
                 credits=0,
                 passmark=40,
                 semester=1, **kw):
        super(Course, self).__init__(**kw)
        self.title = title
        self.code = code
        self.level = level
        self.credits = credits
        self.passmark = passmark
        self.semester = semester
        
class CourseFactory(grok.GlobalUtility):
    """A factory for courses.
    """
    grok.implements(IFactory)
    grok.name(u'waeup.Course')
    title = u"Create a new course.",
    description = u"This factory instantiates new course instances."

    def __call__(self, *args, **kw):
        return Course(*args, **kw)

    def getInterfaces(self):
        return implementedBy(Course)

class DisplayForm(grok.DisplayForm):
    grok.context(ICourse)
    form_fields = grok.AutoFields(ICourse)

    
class Content(FormWrapMixin, grok.Viewlet):
    """A viewlet that wraps the `AddDepartmentForm`.
    """
    grok.viewletmanager(MainArea)
    grok.context(ICourse)
    grok.view(Index)
    grok.require('waeup.manageUniversity')
    grok.order(3)

    formview_name = 'displayform' # The name of the formview we want
                                  # to be rendered in this viewlet.

class ContentHeader(grok.Viewlet):
    """A viewlet that renders the faculty, department and course title.
    """
    grok.viewletmanager(MainArea)
    grok.context(ICourse)
    grok.view(Index)
    grok.order(2)

    def render(self):
        return '''<h5>%s</h5><h4>%s</h4><h3>%s</h3>''' % (
            getName(self.context.__parent__.__parent__.__parent__),
            getName(self.context.__parent__.__parent__),
            self.context.title)
        
class ManageForm(grok.EditForm):
    """Manage the basic properties of a `Course` instance.
    """

    form_fields = grok.AutoFields(ICourse)

    @property
    def label(self):
        # Set department name as form label
        #return self.form_fields['title'].field.get(self.context)
        return ''
    
    @grok.action('Save')
    def save(self, **data):
        self.applyData(self.context, **data)
        return
    
    @grok.action('Save and return')
    def saveAndReturn(self, **data):
        self.applyData(self.context, **data)
        self.redirect(self.url(self.context))
        return

    @grok.action('Cancel')
    def cancel(self, **data):
        self.redirect(self.url(self.context))
        return

class ManageCourse(FormWrapMixin, grok.Viewlet):
    """A viewlet that renders the `ManageForm`.
    """
    grok.viewletmanager(MainArea)
    grok.context(ICourse)
    grok.view(Manage)
    grok.require('waeup.manageUniversity')
    
    formview_name = 'manageform' # The name of the formview we want to
                                 # be rendered in this viewlet.
